==> hlog-0.18.tar.gz: hlog-0.18/common/zones-cq.c <==
/*
 * Copyright (c) 2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*
 * The official definition of CQ zones can be found at:
 * https://cq-amateur-radio.com/cq_awards/cq_waz_awards/cq_waz_list.html
 */

#include <jeffpc/types.h>
#include <jeffpc/array.h>
#include <jeffpc/cstr.h>
#include <jeffpc/error.h>

struct state {
	const char *state;
	enum zone_predicate {
		ZP_UNKNOWN = 0, /* unknown/not implemented */
		ZP_CONST, /* always the same value */
	} pred;
	union {
		int constant; /* ZP_CONST */
	};
};

static const struct state canada[13] = {
	{ "NS", ZP_CONST, .constant = 5 },
	{ "QC", ZP_UNKNOWN }, /* 2 or 5 */
	{ "ON", ZP_UNKNOWN }, /* 2, 4, or 5 */
	{ "MB", ZP_CONST, .constant = 4 },
	{ "SK", ZP_CONST, .constant = 4 },
	{ "AB", ZP_CONST, .constant = 4 },
	{ "BC", ZP_CONST, .constant = 3 },
	{ "NT", ZP_CONST, .constant = 5 },
	{ "NB", ZP_CONST, .constant = 5 },
	{ "NL", ZP_UNKNOWN }, /* Newfoundland is CQ zone 5, Labrador is 2 */
	{ "YT", ZP_CONST, .constant = 1 },
	{ "PE", ZP_CONST, .constant = 5 },
	{ "NU", ZP_CONST, .constant = 5 },
};

static const struct state usa[] = {
	/* call area 1 */
	{ "CT", ZP_CONST, .constant = 5 },
	{ "ME", ZP_CONST, .constant = 5 },
	{ "MA", ZP_CONST, .constant = 5 },
	{ "NH", ZP_CONST, .constant = 5 },
	{ "RI", ZP_CONST, .constant = 5 },
	{ "VT", ZP_CONST, .constant = 5 },

	/* call area 2 */
	{ "NJ", ZP_CONST, .constant = 5 },
	{ "NY", ZP_CONST, .constant = 5 },

	/* call area 3 */
	{ "DE", ZP_CONST, .constant = 5 },
	{ "MD", ZP_CONST, .constant = 5 },
	{ "PA", ZP_CONST, .constant = 5 },

	/* call area 4 */
	{ "AL", ZP_CONST, .constant = 4 },
	{ "FL", ZP_CONST, .constant = 5 },
	{ "GA", ZP_CONST, .constant = 5 },
	{ "KY", ZP_CONST, .constant = 4 },
	{ "NC", ZP_CONST, .constant = 5 },
	{ "SC", ZP_CONST, .constant = 5 },
	{ "TN", ZP_CONST, .constant = 4 },
	{ "VA", ZP_CONST, .constant = 5 },

	/* call area 5 */
	{ "AR", ZP_CONST, .constant = 4 },
	{ "LA", ZP_CONST, .constant = 4 },
	{ "MS", ZP_CONST, .constant = 4 },
	{ "NM", ZP_CONST, .constant = 4 },
	{ "OK", ZP_CONST, .constant = 4 },
	{ "TX", ZP_CONST, .constant = 4 },

	/* call area 6 */
	{ "CA", ZP_CONST, .constant = 3 },

	/* call area 7 */
	{ "AZ", ZP_CONST, .constant = 3 },
	{ "ID", ZP_CONST, .constant = 3 },
	{ "MT", ZP_CONST, .constant = 4 },
	{ "NV", ZP_CONST, .constant = 3 },
	{ "OR", ZP_CONST, .constant = 3 },
	{ "UT", ZP_CONST, .constant = 3 },
	{ "WA", ZP_CONST, .constant = 3 },
	{ "WY", ZP_CONST, .constant = 4 },

	/* call area 8 */
	{ "MI", ZP_CONST, .constant = 4 },
	{ "OH", ZP_CONST, .constant = 4 },
	{ "WV", ZP_CONST, .constant = 5 },

	/* call area 9 */
	{ "IL", ZP_CONST, .constant = 4 },
	{ "IN", ZP_CONST, .constant = 4 },
	{ "WI", ZP_CONST, .constant = 4 },

	/* call area 0 */
	{ "CO", ZP_CONST, .constant = 4 },
	{ "IA", ZP_CONST, .constant = 4 },
	{ "KS", ZP_CONST, .constant = 4 },
	{ "MN", ZP_CONST, .constant = 4 },
	{ "MO", ZP_CONST, .constant = 4 },
	{ "NE", ZP_CONST, .constant = 4 },
	{ "ND", ZP_CONST, .constant = 4 },
	{ "SD", ZP_CONST, .constant = 4 },
};

static int find_state(const char *state, const struct state *infos,
		      size_t ninfos)
{
	const struct state *info;
	size_t i;

	for (i = 0; i < ninfos; i++) {
		info = &infos[i];

		if (strcasecmp(state, info->state))
			continue;

		switch (info->pred) {
			case ZP_UNKNOWN:
				return -1;
			case ZP_CONST:
				return info->constant;
		}

		panic("unknown CQ zone predicate %d", info->pred);
	}

	return -1;
}

int cqz_by_state(int dxcc, const char *state)
{
	switch (dxcc) {
		case 1: /* Canada */
			return find_state(state, canada, ARRAY_LEN(canada));
		case 6: /* Alaska */
			return 1;
		case 110: /* Hawaii */
			return 31;
		case 291: /* United States */
			return find_state(state, usa, ARRAY_LEN(usa));
	}

	return -1;
}

==> hlog-0.22.tar.gz: hlog-0.22/common/zones-cq.c <==
/*
 * Copyright (c) 2021-2022 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*
 * The official definition of CQ zones can be found at:
 * https://cq-amateur-radio.com/cq_awards/cq_waz_awards/cq_waz_list.html
 */

#include "zones_impl.h"

static const struct zone canada[13] = {
	{ "NS", ZP_CONST, .constant = 5 },
	{ "QC", ZP_UNKNOWN }, /* 2 or 5 */
	{ "ON", ZP_UNKNOWN }, /* 2, 4, or 5 */
	{ "MB", ZP_CONST, .constant = 4 },
	{ "SK", ZP_CONST, .constant = 4 },
	{ "AB", ZP_CONST, .constant = 4 },
	{ "BC", ZP_CONST, .constant = 3 },
	{ "NT", ZP_CONST, .constant = 5 },
	{ "NB", ZP_CONST, .constant = 5 },
	{ "NL", ZP_UNKNOWN }, /* Newfoundland is CQ zone 5, Labrador is 2 */
	{ "YT", ZP_CONST, .constant = 1 },
	{ "PE", ZP_CONST, .constant = 5 },
	{ "NU", ZP_CONST, .constant = 5 },
};

static const struct zone usa[] = {
	/* call area 1 */
	{ "CT", ZP_CONST, .constant = 5 },
	{ "ME", ZP_CONST, .constant = 5 },
	{ "MA", ZP_CONST, .constant = 5 },
	{ "NH", ZP_CONST, .constant = 5 },
	{ "RI", ZP_CONST, .constant = 5 },
	{ "VT", ZP_CONST, .constant = 5 },

	/* call area 2 */
	{ "NJ", ZP_CONST, .constant = 5 },
	{ "NY", ZP_CONST, .constant = 5 },

	/* call area 3 */
	{ "DC", ZP_CONST, .constant = 5 }, /* not really a state */
	{ "DE", ZP_CONST, .constant = 5 },
	{ "MD", ZP_CONST, .constant = 5 },
	{ "PA", ZP_CONST, .constant = 5 },

	/* call area 4 */
	{ "AL", ZP_CONST, .constant = 4 },
	{ "FL", ZP_CONST, .constant = 5 },
	{ "GA", ZP_CONST, .constant = 5 },
	{ "KY", ZP_CONST, .constant = 4 },
	{ "NC", ZP_CONST, .constant = 5 },
	{ "SC", ZP_CONST, .constant = 5 },
	{ "TN", ZP_CONST, .constant = 4 },
	{ "VA", ZP_CONST, .constant = 5 },

	/* call area 5 */
	{ "AR", ZP_CONST, .constant = 4 },
	{ "LA", ZP_CONST, .constant = 4 },
	{ "MS", ZP_CONST, .constant = 4 },
	{ "NM", ZP_CONST, .constant = 4 },
	{ "OK", ZP_CONST, .constant = 4 },
	{ "TX", ZP_CONST, .constant = 4 },

	/* call area 6 */
	{ "CA", ZP_CONST, .constant = 3 },

	/* call area 7 */
	{ "AZ", ZP_CONST, .constant = 3 },
	{ "ID", ZP_CONST, .constant = 3 },
	{ "MT", ZP_CONST, .constant = 4 },
	{ "NV", ZP_CONST, .constant = 3 },
	{ "OR", ZP_CONST, .constant = 3 },
	{ "UT", ZP_CONST, .constant = 3 },
	{ "WA", ZP_CONST, .constant = 3 },
	{ "WY", ZP_CONST, .constant = 4 },

	/* call area 8 */
	{ "MI", ZP_CONST, .constant = 4 },
	{ "OH", ZP_CONST, .constant = 4 },
	{ "WV", ZP_CONST, .constant = 5 },

	/* call area 9 */
	{ "IL", ZP_CONST, .constant = 4 },
	{ "IN", ZP_CONST, .constant = 4 },
	{ "WI", ZP_CONST, .constant = 4 },

	/* call area 0 */
	{ "CO", ZP_CONST, .constant = 4 },
	{ "IA", ZP_CONST, .constant = 4 },
	{ "KS", ZP_CONST, .constant = 4 },
	{ "MN", ZP_CONST, .constant = 4 },
	{ "MO", ZP_CONST, .constant = 4 },
	{ "NE", ZP_CONST, .constant = 4 },
	{ "ND", ZP_CONST, .constant = 4 },
	{ "SD", ZP_CONST, .constant = 4 },
};

int cqz_by_state(int dxcc, const char *state)
{
	switch (dxcc) {
		case 1: /* Canada */
			return find_zone(state, canada, ARRAY_LEN(canada));
		case 6: /* Alaska */
			return 1;
		case 110: /* Hawaii */
			return 31;
		case 291: /* United States */
			return find_zone(state, usa, ARRAY_LEN(usa));
	}

	return -1;
}
